import * as redis from "redis";
import { User } from "../../common/types/User";
import dotenv from "dotenv";

const envVars = dotenv.config().parsed;


class Redis {
  client: any;

  async connect(): Promise<void> {
    this.client = redis.createClient({ url: envVars.REDIS_CONNECTION_URL });
    this.client.on("connect", () => {
      console.log("Connected to Redis");
    });

    this.client.on("error", (err) => {
      console.log("Error connecting to Redis:", err);
      process.exit(1);
    });
  }

  public async findUserByKey(key: string) {
    try {
      this.connect();
      return new Promise((resolve, reject) => {
        this.client.hget("users", key, (err: Error | null, userDataJson: string | null) => {
          if (err) {
            console.error("Error finding user in Redis:", err);
            reject(err);
          } else {
            if (userDataJson) {
              const userData: User = JSON.parse(userDataJson);
              resolve(userData);
            } else {
              resolve(null);
            }
          }
        });
      });
    } catch (error) {
      console.error("Error connecting to Redis:", error);
      throw error;
    } finally {
      this.close();
    }
  }
  
  public async deleteUserByKey(key: string) {
    try {
      this.connect();
      return new Promise((resolve, reject) => {
        this.client.hdel("users", key, (err: Error | null, userDataJson: string | null) => {
          if (err) {
            console.error("Error deleting user in Redis:", err);
            reject(err);
          } else {
            console.log("User deleted")
            resolve(null);
          }
        });
      });
    } catch (error) {
      console.error("Error connecting to Redis:", error);
      throw error;
    } finally {
      this.close();
    }
  }


  async close(): Promise<void> {
    await new Promise((resolve) => this.client.quit(resolve));
    console.log("Disconnected from Redis");
  }
}

export { Redis };
