import * as redis from "redis";
import { User } from "../common/types/User";
import { randomUUID } from "crypto";
import dotenv from "dotenv";


class Redis {
  client: any;
  envVars: any;
  
  constructor() {
    this.envVars = dotenv.config().parsed;
    console.log("Redis class is created");
  }

  async connect(): Promise<void> {
    this.client = redis.createClient({ url: this.envVars.REDIS_CONNECTION_URL });
    this.client.on("connect", () => {
      console.log("Connected to Redis");
    });

    this.client.on("error", (err) => {
      console.log("Error connecting to Redis:", err);
      process.exit(1);
    });
  }

  async transactionToRedis(data: User[]): Promise<void> {
    const EXPIRATION_TIME = 60 * 60 * 24 * 4; 
    
    const multi = this.client.multi();
    multi.flushall();

    for (const user in data) {
      if (!data[user].EmployeeID || !data[user].Email || !data[user].Name || !data[user].Phone) {
        this.rollback("Invalid user data", user);
        return;
      }
      data[user].SecureKey = this.randomUUID()
      data[user].Email = this.encrypt(data[user].Email)
      multi.expire("users", EXPIRATION_TIME);
      multi.hset("users", data[user].SecureKey, JSON.stringify(data[user]));
    };

    multi.exec(async (err: any, replies: any) => {
      if (err) {
        console.log("Error inserting data in Redis:", err.message);
      } else {
        console.log("Data inserted in Redis:", replies);
        await this.pushToSqs(data);
      }
    });
  }

  rollback(reason: string, session: any, data = null): void {
    console.error(reason, data);
    session.discard();
  }

  randomUUID(): string {
    return randomUUID();
  }

  private encrypt(email: string): string {
    return email;
  }

  async close(): Promise<void> {
    await new Promise((resolve) => this.client.quit(resolve));
    console.log("Disconnected from Redis");
  }

  private async pushToSqs(data: User[]): Promise<void> {
    console.log("Pushing to SQS");
  }
}

export { Redis };
